@@ -1,4 +1,3 @@
-
 from gevent.event import AsyncResult
 from gevent import Timeout, spawn
 from traceback import format_exc
@@ -12,7 +11,7 @@ class SynchronousStompException(Exception):
 class OutOfSequenceReply(SynchronousStompException):
     pass
 
-class SynchronousStompMessageException(SynchronousStompException):
+class RemoteException(Exception):
     def __init__(self, err_code, message='', origin=None, trace=None):
         self.err_code = err_code
         self.message = str(message)
@@ -21,7 +20,7 @@ class SynchronousStompMessageException(SynchronousStompException):
 
     @classmethod
     def from_json(cls, err_code, msg):
-        if err_code is not SynchronousStompError.EXCEPTION:
+        if err_code is not SynchronousStompErrorCode.EXCEPTION:
             return cls(err_code, msg, trace=[format_exc()])
 
         exc = cls(err_code,
@@ -37,16 +36,16 @@ class SynchronousStompMessageException(SynchronousStompException):
                }
 
     def __str__(self):
-        return ('SynchronousStompMessageException(%d, "%s")\n%s\n%s'
+        return ('RemoteException(%s, "%s")\n%s\n%s'
                 % (self.err_code,
                    self.message,
                    self.origin or '',
                    '\n'.join(self.trace or [])))
 
 def repack_exception(exc):
-    if isinstance(exc, SynchronousStompMessageException):
+    if isinstance(exc, RemoteException):
         trace = exc.trace or []
-        trace.insert(0, format_exc)
+        trace.insert(0, format_exc())
         msg = exc.to_json()
         msg['trace'] = trace
     else:
@@ -56,11 +55,10 @@ def repack_exception(exc):
               }
     return msg
 
-class SynchronousStompError(object):
-    OK = 0
-    TIMEOUT = 1
-    BAD_MSG = 2
-    EXCEPTION = 3
+class SynchronousStompErrorCode(object):
+    TIMEOUT = 'TIMEOUT'
+    BAD_MESSAGE = 'BAD_MESSAGE'
+    EXCEPTION = 'EXCEPTION'
 
 class Conversation(object):
     """The embodiment of a synchronous STOMP conversation.
@@ -80,7 +78,8 @@ class Conversation(object):
         """Handle an inbound conversation message"""
         # Make sure there is an async result waiting for an answer
         if self.async_result is None:
-            logger.error('Unexpected message for conversation %s:%s' % (self.local_id, self.remote_id))
+            logger.error('Unexpected message for conversation %s:%s' %
+                (self.local_id, self.remote_id))
             return
 
         # Record the remote ID
@@ -90,6 +89,7 @@ class Conversation(object):
         if reply_to is None:
             logger.debug('Received conversation termination for %s:%s with %s'
                          % (self.local_id, self.remote_id, repr(self.reply_to)))
+            self.ended = True
 
         # Set the subsequent reply address
         self.reply_to = reply_to
@@ -130,7 +130,8 @@ class Conversation(object):
             final = True
 
         # Prepare to receive a reply
-        if not final: self.async_result = AsyncResult()
+        if not final:
+            self.async_result = AsyncResult()
         else:
             # Cancel registration with the SynchronousStomp object
             self.s_stomp.convs.pop(self.local_id)
@@ -154,9 +155,10 @@ class Conversation(object):
 
         # Wait for a reply
         try:
-            if not final: return self.async_result.get(timeout=timeout)
+            if not final:
+                return self.async_result.get(timeout=timeout)
         except Timeout:
-            self.error(SynchronousStompError.TIMEOUT)
+            self.error(SynchronousStompErrorCode.TIMEOUT)
             raise
 
     def error(self, err_code, message=''):
@@ -164,12 +166,13 @@ class Conversation(object):
         # Build headers
         headers = self._build_headers()
         headers['SynchronousStomp-error'] = err_code
-        logger.error('Sending conversation error message %d: %s' % (err_code, message))
+        logger.error('Sending conversation error message %s: %s' % (err_code, message))
         self.s_stomp.stomp.send(message, destination=self.reply_to, **headers)
 
 class SynchronousStomp(object):
     """A wrapper around syncstomp for permitting synchronous conversations over STOMP"""
     SYNCHRONOUS_STOMP_COUNT = 0
+
     def __init__(self, stomp_connection, destination=None):
         # Record parameters
         self.stomp = stomp_connection
@@ -187,12 +190,13 @@ class SynchronousStomp(object):
 
         # Register listener and subscribe
         self.stomp.set_listener(self.sync_id, self)
-        self.stomp.subscribe(destination=self.destination,id=self.sync_id)
-    
+        self.stomp.subscribe(destination=self.destination, id=self.sync_id)
+
     def on_message(self, headers, message):
         """Handle an inbound synchronous STOMP message"""
         # Drop messages not intended for this subscription
-        if headers['subscription'] != self.sync_id: return
+        if headers['subscription'] != self.sync_id:
+            return
 
         if 'src-conversation' not in headers:
             logger.error('%s: message missing source conversation id' % self.sync_id)
@@ -219,8 +223,8 @@ class SynchronousStomp(object):
             # Determine if there was an exception
             if 'SynchronousStomp-error' in headers:
                 # Build an exception
-                err_code = int(headers['SynchronousStomp-error'])
-                exc = SynchronousStompMessageException.from_json(err_code, message)
+                err_code = headers['SynchronousStomp-error']
+                exc = RemoteException.from_json(err_code, message)
                 logger.error('%s: conversation %s error message: %s'
                              % (self.sync_id, conv_id, str(exc)))
                 conversation._error(exc)
