@@ -7,6 +7,13 @@ from base64 import b64encode, b64decode
 import logging
 logger = logging.getLogger(__name__)
 
+def on_exception_wrap(wrapper, name):
+    def on_exception_wrapper(exc):
+        logger.critical('caught %s exception: %s', name, exc, exc_info=True)
+        if hasattr(wrapper, 'on_exception'):
+            wrapper.on_exception(exc)
+    return on_exception_wrapper
+
 class _ListenerWrapper(object):
     def __init__(self, listener):
         self.listener = listener
@@ -14,6 +21,9 @@ class _ListenerWrapper(object):
         return dir(self.listener)
     def __getattr__(self, name):
         return getattr(self.listener, name)
+    def on_send(self, headers, body):
+        if 'on_send' in dir(self.listener):
+            return self.listener.on_send(headers, body)
     def on_message(self, headers, json_message):
         # Determine the encoding (JSON by default)
         content_type = headers.get('content-type', 'application/json')
@@ -37,7 +47,7 @@ class _ListenerWrapper(object):
         try:
             self.listener.on_message(headers, message)
         except Exception as exc:
-            logger.critical('caught on_message exception: %s' % exc, exc_info=True)
+            on_exception_wrap(self, 'on_message')(exc)
 
 class Connection(RawConnection):
     """A json-wrapped version of stomp.py's Connection object with DNS service
