@@ -1,11 +1,24 @@
+"""
+Infrastructure for making Python classes that provide RPC over STOMP.
+"""
 
-import logging; logger = logging.getLogger(__name__)
-from .conversation import SynchronousStomp, SynchronousStompError, repack_exception
+import logging
+logger = logging.getLogger(__name__)
+from .conversation import SynchronousStomp, SynchronousStompErrorCode, repack_exception
 from functools import wraps
 
+RPC_METHOD_PREFIX = 'rpc_'
+RPC_EXTENDED_METHOD_SUFFIX = '_ext'
+
 def onecall(func):
     """Wrap a method call as a call-and-return RPC dialogue, such that the
        conversation object does not need to be passed to the method"""
+
+    # For compatibility: if the function has been run through onecall already,
+    # just return it
+    if hasattr(func, 'wrapped'):
+        return func
+
     @wraps(func)
     def wrapper(*args, **kwargs):
         conversation = kwargs.pop('conversation')
@@ -14,11 +27,12 @@ def onecall(func):
             result = func(*args, **kwargs)
         except Exception as exc:
             logger.error('caught RPC exception', exc_info=True)
-            conversation.error(SynchronousStompError.EXCEPTION,
+            conversation.error(SynchronousStompErrorCode.EXCEPTION,
                                repack_exception(exc))
             return
         conversation.reply(result, final=True)
 
+    wrapper.wrapped = True
     return wrapper
 
 class RpcProvider(SynchronousStomp):
@@ -28,10 +42,23 @@ class RpcProvider(SynchronousStomp):
            constructor must prepare a self.services dictionary mapping RPC verbs
            to functions that accept at least a 'conversation' parameter."""
         # Configure the underlying conversation engine
-        super(RpcProvider,self).__init__(stomp_connection=stomp_connection, **kwargs)
+        SynchronousStomp.__init__(self,
+            stomp_connection=stomp_connection,
+            **kwargs)
 
-        # Initialize the service directory
+        # Initialize the service directory, including automatically scanning
+        # for methods that begin with 'rpc_'
         self.services = {'.list_services': self._rpc_list_services}
+        for method_name in dir(self):
+            if method_name.startswith(RPC_METHOD_PREFIX):
+                service_name = method_name[len(RPC_METHOD_PREFIX):]
+                if method_name.endswith(RPC_EXTENDED_METHOD_SUFFIX):
+                    service_name = service_name[:-len(RPC_EXTENDED_METHOD_SUFFIX)]
+                    method_func = getattr(self, method_name)
+                else:
+                    method_func = onecall(getattr(self, method_name))
+                self.services[service_name] = method_func
+        logger.info(self.services.keys())
 
         # Save parameters
         self.keep_running = True
@@ -45,7 +72,7 @@ class RpcProvider(SynchronousStomp):
         """(rpc method) Report the list of RPC services offered by this
            provider"""
         return {'services':     self.services.keys(),
-                'services_doc': dict(((k,v.__doc__) for k,v in
+                'services_doc': dict(((k, v.__doc__) for k, v in
                                       self.services.items())),
                }
 
@@ -58,13 +85,15 @@ class RpcProvider(SynchronousStomp):
             args = message['args']
             kwargs = message['kwargs']
         except KeyError:
-            conversation.error(SynchronousStompError.BAD_MSG, 'Malformed RPC request')
+            conversation.error(SynchronousStompErrorCode.BAD_MESSAGE,
+                'Malformed RPC request')
             return
 
         try:
             func = self.services[verb]
         except KeyError:
-            conversation.error(SynchronousStompError.BAD_MSG, 'RPC verb unknown')
+            conversation.error(SynchronousStompErrorCode.BAD_MESSAGE,
+                'RPC verb unknown')
             return
 
         # Perform the operation
@@ -123,11 +152,12 @@ class RpcProxy(object):
             def method_wrap(*args, **kwargs):
                 conv = self._make_conversation()
                 msg = self._build_rpc_request(name, *args, **kwargs)
-                while not conv.ended:
-                    msg = yield conv.reply(msg)
-                    if msg is None:
-                        conv.reply(msg, final=True)
-                        return
+                try:
+                    while not conv.ended:
+                        msg = yield conv.reply(msg)
+                except GeneratorExit:
+                    if not conv.ended:
+                        conv.reply(None, final=True)
         else:
             # Attempting to access a nonexistant service is an error
             if name not in self._services:
@@ -142,4 +172,3 @@ class RpcProxy(object):
         method_wrap.func_name = name
 
         return method_wrap
-
